<?php 
require_once("icwfData.inc.php");
require_once("FormValidation.inc.php");

class DbAdmin implements icwfData {
  private static $noExport = array("passwdInsecure", "passwd");
  private static $notInArray = array("validation", "passwdInsecure", "passwd", "send_email");
  static private $accessRights;
  
  private $validation;
  private $passwdInsecure;
  private $send_email;
  
  private $db_admin_id;
  private $admin_name;
  private $login;
  private $passwd;
  private $email;
  private $ip_addresses;
  private $db_admin_group_ids;
  private $access_rights;
  private $first_login;
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // initialize
  private function initialize() {
    foreach ($this as $key => $value)
      if(in_array($key, array("db_admin_group_ids", "access_rights")))
        $this->$key = array();
      else
        $this->$key = null;
      
    static::$accessRights = array(  
      "kdb_options"=>_("Change K_Db options"),
      "edit_event"=>_("Edit events"),
      "edit_fees"=>_("Edit fees"),
      "add_registrations"=>_('Add registrations'),
      "edit_payment"=>_("Edit modes of payment"),
      "edit_disclaimers"=>_("Edit disclaimers"),
      "edit_salutations"=>_("Edit salutations"),
      "edit_topics"=>_("Edit topics"),
      "edit_views"=>_("Edit views"),
      "edit_admins"=>_("Edit admins"),
      "edit_admingroups"=>_("Edit groups")
    );
    foreach(Plugins::getAddAccessRights() as $name => $text)
      if (!in_array($name, array_keys(static::$accessRights)) && !in_array($text, static::$accessRights))
        static::$accessRights[$name] = $text;
  }
  
  public function __construct(FormValidation $validation=null) {
    $this->initialize();
    
    $this->validation = $validation;
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // security
  static public function getAccessRights($name=null) {
    if($name)
      return static::$accessRights[$name];
    else
      return static::$accessRights;
  }
  
  private function adminHasAccessRight($accessRight) {
    return (($_SESSION['admin']['login']=="root" || in_array($accessRight, $_SESSION['admin']['access_rights'])) ? 1 : 0);
  }

  public function checkPW($passwd) {
    if(!$this->passwd) {
      if (!$passwd) 
        return true;
      else
        return false;
    }
    else {
      if (preg_match('~^\$([^$]+)\$(.{'.CWFTools::$SALT_LENGTH.'})~', $this->passwd, $match)) {
        $hashAlgorithm = $match[1];
        $salt = $match[2];
        $pwHash = CWFTools::getPasswordHash($passwd, $hashAlgorithm, $salt);
      }
      else {
        $pwHash = CWFTools::getPasswordHash($passwd, "md5", "");
        $pwHash = preg_replace('~^\$([^$]+)\$~', "", $pwHash);
      }
    }
    return ($pwHash == $this->passwd);
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // get values
  public function __get($name) {
    if($name == "id") $name = "db_admin_id";
    if (array_key_exists($name, get_class_vars(get_class())) && !in_array($name, self::$noExport))
      return $this->$name;
      
    return null;
  }

  public function getIDName() {
    return "db_admin_id";
  }
  
  public function toArray() {
    $array = array();
    foreach ($this as $key => $value)
      if(!in_array($key, self::$notInArray)) {
        $array[$key] = $value;
      }
      
    return $array;
  }
  
  public function getPostBackData() {
    return array($this->getIDName()=>$this->id, "passwd"=>"", "generate_passwd"=>"");
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////////
  // set values
  public function setValidation(FormValidation $validation) {
    $this->validation = $validation;
  }
  
  public function fromArray($array) {
    foreach ($this as $key => $value) {
      switch ($key) {
        case "validation":
        case "passwdInsecure":
          break;
          
        case "access_rights":
        case "db_admin_group_ids":
          if(isset($array[$key]) && is_array($array[$key]))
            $this->$key = $array[$key];
          else
            $this->$key = array();
          break;
        case "passwd":
          if(isset($array['generate_passwd']) && $array['generate_passwd']) {
            $this->passwdInsecure = CWFTools::generatePassword();
            $this->$key = CWFTools::getPasswordHash($this->passwdInsecure);
          }
          else if(isset($array[$key]) && $array[$key] && isset($array['passwd_repeat']) && $array[$key]==$array['passwd_repeat']) {
            $this->$key = CWFTools::getPasswordHash($array[$key]);
            $this->passwdInsecure = $array[$key];
          }
          else
            $this->$key = null;
          break;
          
        case "first_login":
        case "send_email":
          $this->$key = ((isset($array[$key]) && $array[$key]) ? 1 : 0);
          break;
          
        default:
          if(isset($array[$key]))
            $this->$key = $array[$key];
          else
            $this->$key = null;
          break;
      }
    }

    return $this->validation->validate($array);
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - read
  public function readData(DbConnection $DB, $conditions) {
    if(is_array($conditions)) {
      $whereConditions = array();
      foreach ($conditions as $key => $value)
        if(array_key_exists($key, get_class_vars(get_class())))
          $whereConditions[] = "dba.$key=$value";
      $conditions =  implode(" AND ", $whereConditions);
    }
    $result = $DB->query("SELECT dba.*, group_concat(dbaa.name SEPARATOR ',') access_rights ". 
                         "FROM db_admins dba ".
                         "LEFT JOIN db_admins_access dbaa ON (dbaa.db_admin_id=dba.db_admin_id)".
                         "WHERE {$conditions};");
    
    if(!$DB->errno() && $DB->num_rows($result)==1) {
      $admin = $DB->fetch_array($result);
      foreach ($admin as $property => $value) {
        if(property_exists($this, $property)) {
          switch ($property) {
            case "access_rights":
            case "db_admin_group_ids":
              $this->$property = explode(",", $value);
              break;
            default:
              $this->$property = $value;
              break;
          }
        }
      }
      
      return array(0, "");
    }
    else if($DB->num_rows($result)>1)
      return array(1, _("Username/IP address ambigous. Please contact the administrator."));
    else if($DB->num_rows($result)<1)
      return array(1, _("Username/IP address does not exist."));
    else
      return array($DB->errno(), $DB->error());
  }
  
  static public function readList(DbConnection $DB, $filter="") {
    if($filter!="")
      $sql = "SELECT db_admin_id id, login name FROM db_admins WHERE {$filter} ORDER BY login;";
    else
      $sql = "SELECT db_admin_id id, login name FROM db_admins ORDER BY login;";
    $result = $DB->query($sql);
    
    $list = array();
    while ($admin = $DB->fetch_array($result))
      $list[] = $admin;
      
    return $list;
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - write
  public function changePW(DbConnection $DB) {
    if(!isset($this->db_admin_id) || !$this->db_admin_id ||
       !isset($this->passwd) || !$this->passwd ||
       $this->db_admin_id<>$_SESSION['admin']['db_admin_id']) return;
    
    $result = $DB->query("UPDATE db_admins SET
        passwd='{$this->passwd}',
        first_login=0
      WHERE db_admin_id='{$this->db_admin_id}';");

    $_SESSION['admin']['first_login']=0;
    return array($DB->errno(), $DB->error());
  }

  public function writeData(DbConnection $DB) {
    if(!isset($this->login) || !$this->login) return;
    //Changed admin is in at least one group with changing admin?
    if($this->db_admin_id && $this->db_admin_id!=$_SESSION['admin']['db_admin_id'] && $_SESSION['admin']['login']!="root") {
      if ($existingGroups = $DB->result($DB->query("SELECT db_admin_group_ids FROM db_admins WHERE db_admin_id='{$this->db_admin_id}';"), 0, 0))
        $existingGroups = explode(",", $existingGroups);
      else
        $existingGroups = array();
      if(count($existingGroups)>0 &&
         count(array_intersect($existingGroups, $_SESSION['admin']['db_admin_group_ids']))==0)
        return;
      //Readd existing groups that changing admin is not allowed to change
      $existingGroups = array_diff($existingGroups, $_SESSION['admin']['db_admin_group_ids']);
      $this->db_admin_group_ids = array_merge($this->db_admin_group_ids, $existingGroups);
      sort($this->db_admin_group_ids);
    }
    if(isset($this->db_admin_id) && $this->db_admin_id) {
      $result = $DB->query("UPDATE db_admins SET
          admin_name='{$this->admin_name}',
          login='{$this->login}',
          ".((isset($this->passwd) && $this->passwd) ? "passwd='{$this->passwd}'," : "")."
          email='{$this->email}',
          ip_addresses='{$this->ip_addresses}',
          db_admin_group_ids='".implode(",", $this->db_admin_group_ids)."',
          first_login='{$this->first_login}'
        WHERE db_admin_id='{$this->db_admin_id}';");
    }
    else {
      $result = $DB->query("INSERT INTO db_admins(
          admin_name,
          login,
          passwd,
          email,
          ip_addresses,
          db_admin_group_ids,
          first_login
        ) VALUES(
          '{$this->admin_name}',
          '{$this->login}',
          '{$this->passwd}',
          '{$this->email}',
          '{$this->ip_addresses}',
          '".implode(",", $this->db_admin_group_ids)."',
          '{$this->first_login}'
        );");
      
      $this->db_admin_id = $DB->insert_id();
    }
    if(!$DB->errno()) {
      //Update access rights
      if($_SESSION['admin']['login']=="root")
        $_SESSION['admin']['access_rights'] = array_keys(DbAdmin::getAccessRights());
      $toBeDeleted = array_diff($_SESSION['admin']['access_rights'], $this->access_rights);
      $toBeAdded = array_intersect(array_keys(DbAdmin::getAccessRights()), $_SESSION['admin']['access_rights'], $this->access_rights);
      
      $DB->query("DELETE FROM db_admins_access WHERE db_admin_id='{$this->db_admin_id}' AND name IN ('".implode("', '", $toBeDeleted)."');");
      foreach($toBeAdded as $name)
        $DB->query("INSERT IGNORE INTO db_admins_access(db_admin_id, name) VALUES('{$this->db_admin_id}', '{$name}');");
      
      if($this->send_email && $this->email) {
        $emailText = _("Admin account for")." http://".
                     $_SERVER['SERVER_NAME'].
                     CWF_HTTP_PATH."admin/\n\n".
                     _("User name").": {$this->login}\n".
                     _("Password").": {$this->passwdInsecure}\n"; 
        CWFTools::sendEmail($this->email, $emailText, _("K_Db Administrator Account"), ROOT_EMAIL);
      }
    }
      
    return array($DB->errno(), $DB->error());
  }

  static public function deleteData(DbConnection $DB, $id) {
    if(!preg_match("~^[0-9]+$~", $id))
      return array(1, "ID not valid.");
    //Changed admin is in at least one group with changing admin?
    if($_SESSION['admin']['login']!="root") {
      if ($existingGroups = $DB->result($DB->query("SELECT db_admin_group_ids FROM db_admins WHERE db_admin_id='{$id}';"), 0, 0))
        $existingGroups = explode(",", $existingGroups);
      else
        $existingGroups = array();
      if(count($existingGroups)>0 && count(array_intersect($existingGroups, $_SESSION['admin']['db_admin_group_ids']))==0)
        return array(1, "Not enough privileges.");
    }
    
    $result = $DB->query("DELETE FROM db_admins WHERE db_admin_id='{$id}';");
    $result = $DB->query("DELETE FROM db_admins_access WHERE db_admin_id='{$id}';");
    
    if(!$DB->errno() && $DB->affected_rows()>0)
      return array(0, "");
    else 
      return array($DB->errno(), $DB->error());
  }

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - misc
  static public function createTable(DbConnection $DB) {
    $result = $DB->query("
      CREATE TABLE db_admins (
        db_admin_id int unsigned NOT NULL AUTO_INCREMENT PRIMARY KEY,
        admin_name text,
        login text,
        passwd text,
        email text,
        ip_addresses text,
        db_admin_group_ids text,
        first_login tinyint
      );");
    if ($result) { 
      echo _("&nbsp;&nbsp;- creating table").": 'db_admins'.<br>\n";
      echo _("&nbsp;&nbsp;- creating user: 'root' (login: root, password: root, has to be changed on first login).")."<br>\n";
      $result = $DB->query("INSERT INTO db_admins(
          admin_name, login, passwd, email, ip_addresses, db_admin_group_ids, first_login
        ) VALUES(
          'root', 'root', '".CWFTools::getPasswordHash('root')."', '', '', '', 1
        );");
      $adminId = $DB->insert_id();
    }
    else { echo _("&nbsp;&nbsp;<b>- Error when creating table</b>")." 'db_admins'<br>\n"; $error = 1; }

      $result = $DB->query("
      CREATE TABLE db_admins_access (
        db_admin_id int unsigned NOT NULL,
        name varchar(100) NOT NULL,
        PRIMARY KEY(db_admin_id, name),
        INDEX (db_admin_id)
      );");
    if ($result) { 
      echo _("&nbsp;&nbsp;- creating table").": 'db_admins_access'.<br>\n";
      echo _("&nbsp;&nbsp;- creating user: 'root'.")."<br>\n";
      $result = $DB->query("INSERT INTO db_admins_access(
          db_admin_id, name
        ) VALUES
        ( $adminId, 'kdb_options' ),
        ( $adminId, 'edit_event' ),
        ( $adminId, 'edit_fees' ),
        ( $adminId, 'edit_payment' ),
        ( $adminId, 'edit_disclaimers' ),
        ( $adminId, 'edit_salutations' ),
        ( $adminId, 'edit_topics' ),
        ( $adminId, 'edit_views' ),
        ( $adminId, 'edit_admins' ),
        ( $adminId, 'edit_admingroups' );");
    }
    else { echo _("&nbsp;&nbsp;<b>- Error when creating table</b>")." 'db_admins_access'<br>\n"; $error = 1; }
  }
}
?>