<?php
abstract class cwfUI {
  protected static $accessRight = "";
  protected static $dataClass = "";
  protected $title;
  protected $header;
  protected $data;
  protected $error;
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // initialize
  public function __construct() {
  }
  
  protected function initialize() {
    $this->title = "";
    if(static::$dataClass) {
      $this->data = new static::$dataClass(static::getValidationRules());
      if($this->data->validation->hasRules())
        $this->header = "  <script type=\"text/javascript\">
    $(document).ready(function(){
      $(\"form\").validate({
        errorClass: 'invalid',\n".
        $this->data->validation->getValidationRulesJQuery("        ")."
      });
    });
  </script>\n";
      else
        $this->header = "";
    }
    else {
      $this->data = null;
      $this->header = "";
    }
    $this->error = "";
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // security
  static public function hasAccessRight($user) {
    if($user['login']=="root" || static::$accessRight=="") return true;
    return (isset($user['access_rights']) && in_array(static::$accessRight, $user['access_rights']));
  }
  
  static public function checkPrivileges($user, $action) {
    if(!static::hasAccessRight($user))
      return _("You do not have enough privileges.");
    
    return;
  }
  
  static public function isAllowedAction($action) {
    return in_array($action, array("updateForm", "save", "saveasnew", "delete"));
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // handle data
  public function setData($data) {
    if (is_a($data, get_class($this->data)))
      $this->data = $data;
    else if (is_array($data))
      $this->data->fromArray($data);
  }
  
  public function getData($asArray=true) {
    if(!$asArray) {
      $data = clone $this->data;
      return $data;
    }
    return $this->data->toArray();
  }
  
  public function getIDName() {
    return $this->data->getIDName();
  }
  
  public function getId() {
    return $this->data->id;
  }
  
  public function getErrorMessage() {
    return $this->error;
  }
  
  static public function getValidationRules() {
    $validationRules = new FormValidation(array(), array());
    return $validationRules;
  }
  
  public function isValid($action) {
    if(in_array($action, array("updateForm", "delete")))
      return true;
    return $this->data->validation->isValid();
  }
  
  public function getInvalidFields() {
    return $this->data->validation->getInvalidMessages();
  }
  
  protected function getNewValuesXML($newData) {
    $xml = "  <newValues>\n";
    foreach ($newData as $key => $value) {
      if($key=="passwd") {
        $xml .= "    <newValue name=\"$key\"></newValue>\n";
        $xml .= "    <newValue name=\"{$key}_repeat\"></newValue>\n";
      }
      else if(is_array($value)) {
        $xml .= "    <newValue name=\"$key\">\n      <value>".implode("</value>\n      <value>", $value)."</value>\n    </newValue>\n";
      }
      else
        $xml .= "    <newValue name=\"$key\">$value</newValue>\n";
    }
    $xml .= "  </newValues>\n";
    
    return $xml;
  }
  
  public function getPostBackXML(DbConnection $DB, $action) {
    $xml = "";
    switch ($action) {
      case "updateForm":
        $xml .= $this->getNewValuesXML($this->getData());
        break;
      case "saveasnew":
      case "save":
        $xml .= $this->getNewValuesXML($this->data->getPostBackData());
        $xml .= "  <selectList>\n";
        $selectList = static::readList($DB);
        foreach ($selectList as $entry) {
          $xml .= "    <entry>\n";
          $xml .= "      <id>{$entry['id']}</id>\n";
          $xml .= "      <name>{$entry['name']}</name>\n";
          $xml .= "    </entry>\n";
        }
        $xml .= "  </selectList>\n";
        break;
      case "delete":
        $xml .= $this->getNewValuesXML($this->getData());
        $xml .= "  <selectList>\n";
        $selectList = static::readList($DB);
        foreach ($selectList as $entry) {
          $xml .= "    <entry>\n";
          $xml .= "      <id>{$entry['id']}</id>\n";
          $xml .= "      <name>{$entry['name']}</name>\n";
          $xml .= "    </entry>\n";
        }
        $xml .= "  </selectList>\n";
        break;
    }
    return $xml;
  }
  
  public function doAction(DbConnection $DB, $action) {
    $id = "";
    switch ($action) {
      case "updateForm":
        $id = $_POST["id"];
        if($id>=0) {
          list($errno, $error) = $this->readData($DB, array($this->getIDName()=>$_POST["id"]));
          if ($errno)
            $this->error = _("Error")." {$errno}: {$error}";
        }
        break;
      case "saveasnew":
        if(isset($_POST[$this->getIDName()])) unset($_POST[$this->getIDName()]);
      case "save":
        $this->saveForm($DB);
        $id = $this->getId();
        break;
      case "delete":
        $id = $_POST[$this->getIDName()];
    
        list($errno, $error) = $this->deleteData($DB, $id);
        if ($errno>0)
          $errorMsg = _("Error")." {$errno}: {$error}";
        else {
          $this->initialize();
          $id = -1;
        }
        break;
    }
    return $id;
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // Database operations
  public function readData($DB, $conditions) {
    return $this->data->readData($DB, $conditions);
  }
  
  static public function readList($DB) {
    if (static::$dataClass=="") return array();
    $dataClass = static::$dataClass;
    return $dataClass::readList($DB);
  }
  
  public function saveForm(DbConnection $DB) {
    if(isset($_SESSION['admin']) && !static::hasAccessRight($_SESSION['admin'])) {
      $this->error = _("You do not have enough privileges.");
      return 0;
    }
    if($this->data->fromArray($_POST)) {
      list($errno, $error) = $this->data->writeData($DB);
      if ($errno) {
        $this->error = $error;
        return 0;
      }
      return 1;
    }
    $this->error = _("Validation failed.");
    return 0;
  }
  
  public function deleteData(DbConnection $DB, $id) {
    return $this->data->deleteData($DB, $id);
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // output
  public function getTitle() {
    return $this->title;
  }
  public function getHeader() {
    return $this->header;
  }

  public function getHTML(DbConnection $DB) {
  }
}
?>